"use strict";
/*
 * SonarQube JavaScript Plugin
 * Copyright (C) 2011-2024 SonarSource SA
 * mailto:info AT sonarsource DOT com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the Sonar Source-Available License Version 1, as published by SonarSource SA.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Sonar Source-Available License for more details.
 *
 * You should have received a copy of the Sonar Source-Available License
 * along with this program; if not, see https://sonarsource.com/license/ssal/
 */
// https://sonarsource.github.io/rspec/#/rspec/S5743/javascript
Object.defineProperty(exports, "__esModule", { value: true });
exports.rule = void 0;
const index_js_1 = require("../helpers/index.js");
const meta_js_1 = require("./meta.js");
const MESSAGE = 'Make sure allowing browsers to perform DNS prefetching is safe here.';
exports.rule = {
    meta: (0, index_js_1.generateMeta)(meta_js_1.meta, undefined, true),
    create(context) {
        return {
            CallExpression: (node) => {
                const callExpression = node;
                const fqn = (0, index_js_1.getFullyQualifiedName)(context, callExpression);
                if (fqn === 'helmet.dnsPrefetchControl') {
                    (0, index_js_1.checkSensitiveCall)(context, callExpression, 0, 'allow', true, MESSAGE);
                }
                if (fqn === 'helmet') {
                    (0, index_js_1.checkSensitiveCall)(context, callExpression, 0, 'dnsPrefetchControl', false, MESSAGE);
                }
                if (fqn === 'dns-prefetch-control') {
                    (0, index_js_1.checkSensitiveCall)(context, callExpression, 0, 'allow', true, MESSAGE);
                }
            },
        };
    },
};
